#include "stm32f407xx.h"
#include "sysclock.h" 

//_________________GPIO DEFINITION_____________________//
/* enum class otype_t
 * Содержит перечисление типов выходного каскада GPIO
 * pp - Push-Pull
 * od - Open-Drain
 */
enum otype_t
{
  pp = 0,
  od = 1,
};
	
/* enum class ospeed_t
 * Содержит перечисление скоростей выходного каскада GPIO
 * low       -   2 MHz
 * medium    -  10 MHz
 * high      -  10 < x <50 MHz
 * very_high - >50 MHz
*/
enum ospeed_t
{
  low       = 0,
  medium    = 1,
  high      = 2,
  very_high = 3,
};


/* enum class pull_t
 * Содержит перечисление типов подтяжек выходного каскада GPIO
 * floating  -  плавающий выход
 * up        -  подтяжка к питанию
 * down      -  подтяжка к земле
*/
enum  pull_t
{
  floating   = 0,
  up         = 1,
  down       = 2,
};

/* enum class af
 * Содержит перечисление альтернативных функций порта GPIO
*/
enum af_t
{
  system = 0,
  tim1 = 1,    
  tim2 = 1,
  tim3 = 2,
  tim4 = 2,
  tim5 = 2,
  tim8 = 3,
  tim9 = 3,
  tim10 = 3,
  tim11 = 3,
  i2c1 = 4,
  i2c2 = 4,
  i2c3 = 4,
  spi1 = 5,
  spi2 = 5,
  spi3 = 6,
  usart1 = 7,
  usart2 = 7,
  usart3 = 7,
  usart4 = 8,
  usart5 = 8,
  usart6 = 8,
  can1 = 9,
  can2 = 9,
  tim12 = 9,
  tim13 = 9,
  tim14 = 9,
  otg_fs = 10,
  otg_hs1 = 10,
  eth = 11,
  fsmc = 12,
  sdio = 12,
  otg_hs2 = 12,
  dcmi = 13,
  af14 = 14,
  eventout = 15,
};

/* enum class exti_trigger
 * EXTI Trigger list
*/
enum exti_trigger
{
  falling,
  rising,	
  both,		
};
          
          
          
void pincfg_output(GPIO_TypeDef* port, uint8_t pin, uint8_t otype, uint8_t ospeed, uint8_t pull)
{
  // Оригинальный код был написан на C++ с использованием enum class
  // поэтому проверки входных значений не требовались
	
  // 1 - Настройка режима работы
    // 1.1 - Очистка конфигурационных битов
  port->MODER &= ~(3 << (pin << 1));
  // 1.2 - Режим "output"
  port->MODER |= 1 << (pin << 1);
		
  // 2 - Настройка типа выхода
  // 2.1 - Очистка конфигурационных битов
  port->OTYPER &= ~(1 << pin);
  // 2.2 - Установка типа выхода (Push-Pull или Open-Drain)
  port->OTYPER |=  otype << pin;
	  
  // 3 - Настройка скорости
  // 3.1 - Очистка конфигурационных битов
  port->OSPEEDR &= ~(0x03 << (pin << 1));
  // 3.2 - Установка нового значения
  port->OSPEEDR |= ospeed << (pin << 1);
		
  // 4 - Установка подтяжки
  // 4.1 - Очистка конфигурационных битов
  port->PUPDR &= ~(0x03 << (pin << 1));
  // 4.2 - Установка нового значения
  port->PUPDR |= pull << (pin << 1);
			
  // 5 - Сброс альтернативной функции
  if (pin < 8)
    port->AFR[0] &= ~(0x0F << (pin << 2));
  else
    port->AFR[1] &= ~(0x0F << ((pin - 8) << 2));
			
  return;
};


void pincfg_input(GPIO_TypeDef* port, uint8_t pin, uint8_t pull)
{
  // 1 - Настройка режима работы
    // 1.1 - Очистка битов (режим input)
    port->MODER &= ~(3 << (pin << 1));
		
  // 2 - Настройка типа выхода
    // 2.1 - Очистка битов
    port->OTYPER &= ~(1 << pin);
	  
  // 3 - Настройка типа скорости
    // 3.1 - Очистка битов
    port->OSPEEDR &= ~(0x03 << (pin << 1));
		
  // 4 - Выбираем подтяжку
    // 4.1 - Очистка битов
    port->PUPDR &= ~(0x03 << (pin << 1));
    // 4.2 - Установка типа
    port->PUPDR |= pull << (pin << 1);
			
  // 5 - Сброс альтернативной функции
  if (pin < 8)
    port->AFR[0] &= ~(0x0F << (pin << 2));
  else
    port->AFR[1] &= ~(0x0F << ((pin - 8) << 2));
			
  return;
};


void pincfg_analog(GPIO_TypeDef* port, uint8_t pin, uint8_t pull)
{ 
  // 1 - Настройка режима работы
    // 1.1 - Очистка битов 
    port->MODER &= ~(3 << (pin << 1));
  // 1.2 - Установка режима "Analog"
    port->MODER |= 3 << (pin << 1);
		
  // 2 - Настройка типа выхода
    // 2.1 - Очистка битов
    port->OTYPER &= ~(1 << pin);
	  
  // 3 - Настройка типа скорости
    // 3.1 - Очистка битов
    port->OSPEEDR &= ~(0x03 << (pin << 1));
		
  // 4 - Выбираем подтяжку
    // 4.1 - Очистка битов
    port->PUPDR &= ~(0x03 << (pin << 1));
    // 4.2 - Установка типа
    port->PUPDR |= pull << (pin << 1);
		
  // 5 - Сброс альтернативной функции
  if (pin < 8)
    port->AFR[0] &= ~(0x0F << (pin << 2));
  else
    port->AFR[1] &= ~(0x0F << ((pin - 8) << 2));		
			
  return;
};


void pincfg_alternate(GPIO_TypeDef* port, uint8_t pin, uint8_t af, uint8_t otype, uint8_t ospeed, uint8_t pull)
{	
  // 1 - Настройка режима работы
    // 1.1 - Очистка битов
    port->MODER &= ~(0x03 << (pin << 1));
    // 1.2 - Установка режима "AF"
    port->MODER |= 0x02<<(pin << 1);

  // 2 - Настройка типа выхода
    // 2.1 - Очистка битов
    port->OTYPER &= ~(1 << pin);
    // 2.2 - Установка типа
    port->OTYPER |= otype << pin;
	  
  // 3 - Настройка скорости пина
    // 3.1 - Очистка битов
    port->OSPEEDR &= ~(0x03 << (pin << 1));
    // 3.2 - Установка типа
    port->OSPEEDR |= ospeed << (pin << 1);
	
  // 4 - Выбираем подтяжку
    // 4.1 - Очистка битов
    port->PUPDR &= ~(0x03 << (pin << 1));
    // 4.2 - Установка типа
    port->PUPDR |= pull << (pin << 1);
			
  // 5 - Выбор альтернативной функции
  if (pin < 8)
  {
    port->AFR[0] &= ~(0x0F << (pin << 2));
    port->AFR[0] |= af << (pin << 2);
  } else
  {
    port->AFR[1] &= ~(0x0F << ((pin - 8) << 2));
    port->AFR[1] |= af << ((pin - 8) << 2);
  }				
		
  return;
};



/* void pincfg_exti(gpio_t& gpio, exti_trigger trig)
 * Описание: Настройка EXTI для GPIO
 * Параметры: gpio            - ножка мк,
 *            exti_trigger    - тип триггера (RISING/FALLING/BOTH)
 * Note: Дополнительно необходимо включить прерывание в NVIC и создать обработчик.
 *       Не забудьте затактовать SYSCFG (APB2)
*/
void pincfg_exti(GPIO_TypeDef* gpio, uint8_t pin, uint8_t trig)
{
  // Определяем условный номер порта
  uint16_t port  = (uint16_t)(((uint32_t)gpio - (uint32_t)GPIOA)>>10);
  // Определяем номер группы (регистра) 
  uint8_t grp   = pin >> 2;
  // Определяем битовое смещение
  uint8_t offset = (pin - (grp << 2)) << 2;
	
  // Вешаем прерывание на нужный порт
  SYSCFG->EXTICR[grp] |= port<<offset;
	
  // Разрешаем прерывание EXTI
  EXTI->IMR |= (1 << pin);   
	
  if ((trig == falling)||(trig == both))
  { // Прерывание по спаду
    EXTI->FTSR |= (1 << pin);
  }
  if ((trig == rising)||(trig == both))
  {// Прерывание по нарастанию		
    EXTI->RTSR |= (1 << pin);
  }
	
  return;
}
//___________________ GPIO DEFINITION END _____________________//


void timer_init();
int main()
{
  // Настройка тактовой частоты
  // Это ужасно старый кусок кода, но вроде бы он работал.
  // Все давно переписано на шаблоны
  sysclk_config();
  
  // Тактируем порт GPIOA
  RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;
  
  // Я мигаю ногой PA6 
  pincfg_output(GPIOA, 6, pp, low, floating);
  
  // Настройка таймера
  timer_init();
  TIM2->CR1 |= TIM_CR1_CEN;
  
  while(1)
  {
   
  }
}




// При частоте тактирования таймера 84 000 000
// ARR = 20000
// PSC = 2100
// Переполнение будет происходить с периодом 0.5с
void timer_init()
{
  RCC->APB1ENR |= RCC_APB1ENR_TIM2EN;
  
  // 0 - Запрещаем прерывания по обновлению
  TIM2->DIER &= ~TIM_DIER_UIE; 
	
  // 1 - Счет запрещён
  TIM2->CR1 &= ~TIM_CR1_CEN;
   
  // 2 - Установка предделителя и границы счета
  TIM2->ARR = 20000;
  TIM2->PSC = 2100;
	
  // 3 - Установка параметров таймера-счетчика 
  TIM2->CR1 &= ~TIM_CR1_DIR;

  // 4 - Generate Update Event
  TIM2->EGR |= TIM_EGR_UG;    
	
  // 5 - Сброс флага прерывания
  TIM2->SR &= ~TIM_SR_UIF;
	
  // 6 - Буферизация TIM->ARR
  TIM2->CR1 |= TIM_CR1_ARPE;  
  
  // 7 - Активация прерываний
  NVIC_EnableIRQ(TIM2_IRQn);
  TIM2->DIER |= TIM_DIER_UIE; 
  
  return;
}




void TIM2_IRQHandler()
{
  // Прерывание по переполнению таймера
  if (TIM2->SR & TIM_SR_UIF)
  {
    TIM2->SR &= ~TIM_SR_UIF;
    
    // С регистром ODR нужно обращаться крайне нежно!!!
    GPIOA->ODR ^= 1<<6;
    
    TIM2->CR1 |= TIM_CR1_CEN;     // Пуск таймера
  }
  
  return;
}



